<?php

function peachpay_variation_name_rest( WP_REST_Request $request ) {
    return peachpay_variation_name( $request['id'] );
}

function peachpay_variation_details_rest( WP_REST_Request $request ) {
    $variation_id = $request['id'];
    $product = wc_get_product( $variation_id );
    return array(
        'product_id' => $product->get_parent_id(),
        'variation_id' => $variation_id,
        'variation_title' => peachpay_variation_name( $variation_id ),
        'price' => peachpay_get_price( $product ),
    );
}

function peachpay_variation_name( $id ) {
    $product = wc_get_product( $id );
    if ( ! $product ) {
        return '';
    }
    return wc_get_formatted_variation( $product, true, false );
}

function peachpay_product_price_rest( $data ) {
    $currency = $data->get_param( 'currency' );
    $price = peachpay_get_price( wc_get_product( $data['id'] ) );
    return peachpay_compatibility_for_currency_switchers( $price, $currency );
}

/**
 * peachpay_pickup_hours_rest returns the store's timings which the server
 * will use to generate available pickup slots.
 */
function peachpay_pickup_hours_rest() {
    $options = get_option( 'peachpay_options' );

    return array(
        'sunday' => array('open' => $options['Sunday_open'], 'close' => $options['Sunday_close']),
        'monday' => array('open' => $options['Monday_open'], 'close' => $options['Monday_close']),
        'tuesday' => array('open' => $options['Tuesday_open'], 'close' => $options['Tuesday_close']),
        'wednesday' => array('open' => $options['Wednesday_open'], 'close' => $options['Wednesday_close']),
        'thursday' => array('open' => $options['Thursday_open'], 'close' => $options['Thursday_close']),
        'friday' => array('open' => $options['Friday_open'], 'close' => $options['Friday_close']),
        'saturday' => array('open' => $options['Saturday_open'], 'close' => $options['Saturday_close']),
    );
}

/**
 * peachpay_wc_cart_for_rest forces the cart class to load in the REST endpoints
 * because it normally does not. It also clears out the cart in case it has
 * already been loaded.
 */
function peachpay_wc_cart_for_rest() {
    include_once WC_ABSPATH . 'includes/wc-cart-functions.php';
    include_once WC_ABSPATH . 'includes/class-wc-cart.php';
    if ( is_null( WC()->cart ) ) {
        wc_load_cart();
    }
    WC()->cart->empty_cart();
}

/**
 * peachpay_tax_rest returns the tax information for a given location and
 * set of products.
 */
function peachpay_tax_rest( WP_REST_Request $request ) {
    // There are hooks that echo HTML upon actions like adding to cart or applying coupons.
    // ob_start() is an output buffering hack where we buffer the output and then
    // discard it with ob_get_clean() so that we don't break the REST API by
    // outputting HTML along with the response.
    ob_start();

    // For some reason this part has to come before the cart reset, otherwise the
    // coupons will fail to save
    $coupons = array();
    if ( isset( $request['coupons'] ) ) {
        foreach ( $request['coupons'] as $coupon_code => $amount ) {
            $coupon = new WC_Coupon();
            $coupon->set_code( $coupon_code );
            $coupon->set_discount_type( 'fixed_cart' );
            $coupon->set_amount( $amount );
            $coupon->save();
            array_push( $coupons, $coupon_code );
        }
    }

    peachpay_wc_cart_for_rest();

    WC()->customer->set_shipping_location(
        $request['country'],
        $request['state'],
        $request['postcode'],
        $request['city']
    );

    foreach ( $request['products'] as $product ) {
        // The validation added here for WC 5.2.0 doesn't allow passing in the
        // product_id as the variation_id like we have been doing
        // https://github.com/woocommerce/woocommerce/commit/918750fffbbb36974190efeeb4ae0de280756741
        // Set variation_id to the default value in this case
        if ( $product['product_id'] === $product['variation_id'] ) {
            $product['variation_id'] = 0;
        }

        WC()->cart->add_to_cart(
            $product['product_id'],
            intval( $product['quantity'] ),
            $product['variation_id']
        );
    }

    foreach ( $coupons as $coupon_code ) {
        WC()->cart->apply_coupon( $coupon_code );
    }

    $tax = WC()->cart->get_total_tax();
    $to_currency = $request['overriddenCurrency'] ?? null;
    $tax = peachpay_compatibility_for_currency_switchers( $tax, $to_currency );

    ob_get_clean();

    return $tax;
}

function peachpay_compatibility_for_currency_switchers( $amount, $to_currency ) {
    if ( ! isset( $to_currency ) || $to_currency === '' ) {
        return $amount;
    }
    if ( is_plugin_active( 'currency-switcher-woocommerce/currency-switcher-woocommerce.php' ) ) {
        return peachpay_currency_switcher_woocommerce_convert( $amount, $to_currency );
    }
    if ( is_plugin_active( 'woocommerce-jetpack/woocommerce-jetpack.php' ) || is_plugin_active( 'booster-plus-for-woocommerce/booster-plus-for-woocommerce.php' ) ) {
        return peachpay_booster_for_woocommerce_currency_convert( $amount, $to_currency );
    }
    return $amount;
}

function peachpay_currency_switcher_woocommerce_convert( $amount, $to_currency ) {
    if ( 'yes' !== get_option( 'alg_wc_currency_switcher_enabled', 'yes' ) ) {
        return $amount;
    }

    $result = $amount;
    $currency_based_on_address = alg_get_current_currency_code();

    // The plugin has a switcher widget where the user can change the currency.
    // The REST API endpoint has no knowledge of their session, so we must receieve
    // that data explicitly.
    if ( $to_currency !== $currency_based_on_address && $to_currency !== '' ) {
        // Undo the applied exchange rate that was incorrectly assumed
        $result /= alg_wc_cs_get_currency_exchange_rate( $currency_based_on_address );
        // Apply the exchange rate that the customer selected on the screen
        $result *= alg_wc_cs_get_currency_exchange_rate( $to_currency );
    }

    return round( $result, 2 );
}

function peachpay_booster_for_woocommerce_currency_convert( $amount, $to_currency ) {
    $result = $amount;
    if ( $to_currency !== '' ) {
        $result *= wcj_get_currency_exchange_rate( 'multicurrency', $to_currency );
    }
    return round( $result, 2 );
}

/**
 * peachpay_shipping_options_rest returns the shipping options for a given
 * location and set of products.
 */
function peachpay_shipping_options_rest( WP_REST_Request $request ) {
    ob_start();

    peachpay_wc_cart_for_rest();

    $at_least_one_item_has_vendor = false;
    foreach ( $request['products'] as $product ) {
        WC()->cart->add_to_cart( $product['variation_id'], $product['quantity'] );
        if ( isset( $product['vendor_id'] ) ) {
            $at_least_one_item_has_vendor = true;
        }
    }

    $params = array (
        'address' => $request['address'],
        'city' => $request['city'],
        'state' => $request['state'],
        'country' => $request['country'],
        'postcode' => $request['postcode'],
    );
    WC()->shipping->calculate_shipping( get_shipping_packages( $params ) );
    $shipping_methods = WC()->shipping->packages;
    $overridden_currency = $request['overriddenCurrency'] ?? null;

    if ($at_least_one_item_has_vendor) {
        return peachpay_collect_vendor_shipping_options($shipping_methods, $overridden_currency);
    }

    ob_get_clean();

    return peachpay_collect_shipping_options($shipping_methods, $overridden_currency);
}

function peachpay_collect_vendor_shipping_options($shipping_methods, $overridden_currency) {
    $shipping_options_all_vendors = array();
    foreach ( $shipping_methods as $vendor_id => $vendor_shipping_method ) {
        $shipping_options = array();
        foreach ( $vendor_shipping_method['rates'] as $shipping_method ) {
            $cost = $shipping_method->cost;
            if ( is_plugin_active( 'currency-switcher-woocommerce/currency-switcher-woocommerce.php' ) ) {
                $cost = strval( round( peachpay_compatibility_for_currency_switchers( $cost, $overridden_currency ), 2) );
            }
            $shipping_options[] = array(
                'method_id' => $shipping_method->method_id,
                'method_title' => $shipping_method->label,
                'total' => $cost,
            );
        }
        $shipping_options_all_vendors[$vendor_id] = $shipping_options;
    }
    return $shipping_options_all_vendors;
}

function peachpay_collect_shipping_options($shipping_methods, $overridden_currency) {
    $shipping_options = array();
    foreach ( $shipping_methods[0]['rates'] as $full_method_id => $shipping_method ) {
        $cost = $shipping_method->cost;
        if ( is_plugin_active( 'currency-switcher-woocommerce/currency-switcher-woocommerce.php' ) ) {
            $cost = strval( round( peachpay_compatibility_for_currency_switchers( $cost, $overridden_currency ), 2) );
        }
        // we use full_method_id and not $shipping_method->method_id because the former
        // includes a "sub" ID which is necessary if there is more than one flat_rate
        // shipping, for example
        $shipping_options[] = array(
            'method_id' => $full_method_id,
            'method_title' => $shipping_method->label,
            'total' => $cost,
        );
    }
    return $shipping_options;
}

function get_shipping_packages( $params ) {
    // Packages for storing "carts"
    // This was copied from somewhere and I really don't understand how this WC
    // shipping stuff works, but it has worked perfectly since the start!
    $packages = array();
    $packages[0]['contents']                = WC()->cart->cart_contents;
    // $packages[0]['contents_cost']           = null;
    $packages[0]['applied_coupons']         = WC()->session->applied_coupon;
    $packages[0]['destination']['address']  = $params['address'];
    $packages[0]['destination']['city']     = $params['city'];
    $packages[0]['destination']['state']    = $params['state'];
    $packages[0]['destination']['country']  = $params['country'];
    $packages[0]['destination']['postcode'] = $params['postcode'];
    // $packages[0]['destination']['address_2']= '';

    return apply_filters( 'woocommerce_cart_shipping_packages', $packages );
}

function peachpay_cart_fees_rest( WP_REST_Request $request ) {
    ob_start();

    peachpay_wc_cart_for_rest();
    WC()->customer->set_shipping_location(
        $request['address']['country'],
        $request['address']['state'],
        $request['address']['postcode'],
        $request['address']['city']
    );
    foreach ( $request['items'] as $item ) {
        WC()->cart->add_to_cart( $item['product_id'], $item['quantity'] );
    }

    ob_get_clean();

    return WC()->cart->get_fees();
}

function peachpay_upload_rest( WP_REST_Request $request ) {
    $tmp = $request->get_file_params();
    $files = $tmp[ 'files' ];
    if ( empty( $tmp ) || empty( $files ) ) {
        return 'No file provided';
    }
    $file_names = [];
    for ( $i = 0; $i < count( $files[ 'name' ] ); $i++ ) {
        array_push( $file_names, peachpay_process_upload( $files, $i ) );
    }
    return $file_names;
}

function peachpay_process_upload( $files, $i ) {
    $permittedTypes = ['image/png', 'image/jpeg', 'image/webp'];
    try {
        // smoke/sanity check
        if ( ! $files[ 'name' ][ $i ] ) {
            throw new Exception( 'Error' );
        }
        // confirm file uploaded via POST
        if ( ! is_uploaded_file( $files[ 'tmp_name' ][ $i ] ) ) {
            throw new Exception( 'File upload check failed ');
        }
        // confirm no file errors
        if ( ! $files[ 'error' ][ $i ] === UPLOAD_ERR_OK ) {
            throw new Exception( 'Upload error: ' . $files[ 'error' ][ $i ] );
        }
        // check type
        $mimeType = mime_content_type( $files[ 'tmp_name' ][ $i ] );
        if ( ! in_array( $files[ 'type' ][ $i ], $permittedTypes )
            || ! in_array( $mimeType, $permittedTypes ) ) {
                throw new Exception( 'Invalid file type' );
        }
        } catch ( Exception $e ) {
        return $e->getMessage();
        }

        if ( ! function_exists( 'wp_handle_upload' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
        }

        $upload_overrides = array(
            'test_form' => false
        );

        $fileToMove = array(
        'name' => $files[ 'name' ][ $i ],
        'type' => $files[ 'type' ][ $i ],
        'tmp_name' => $files[ 'tmp_name' ][ $i ],
        'error' => $files[ 'error' ][ $i ],
        'size' => $files[ 'size' ][ $i ],
        );

        $movefile = wp_handle_upload( $fileToMove, $upload_overrides );

        if ( $movefile && ! isset( $movefile['error'] ) ) {
            return $movefile['url'];
        } else {
            /*
            * Error generated by _wp_handle_upload()
            * @see _wp_handle_upload() in wp-admin/includes/file.php
            */
            echo $movefile['error'];
        }
}

function peachpay_woo_discount_rules_discount_rest( WP_REST_Request $request ) {
    return peachpay_woo_discount_rules_get_discount( $request[ 'id' ] );
}

function peachpay_woo_discount_rules_get_discount( $product_id ) {
    if ( is_plugin_active( 'woo-discount-rules/woo-discount-rules.php' ) ) {
        include_once( WP_PLUGIN_DIR . '/woo-discount-rules/woo-discount-rules.php' );
        $manage_discount = new Wdr\App\Controllers\ManageDiscount();
        $discount_calculator = new Wdr\App\Controllers\DiscountCalculator($manage_discount->getDiscountRules());
        $discount_info = $discount_calculator->getProductPriceToDisplay((int) $product_id, 1);
        if ( ! $discount_info[ 'discounted_price' ] ) {
            return false;
        }
        return $discount_info[ 'discounted_price' ];
    }
    return false;
}

function peachpay_check_plugin_status() {
    return array (
        'merchantName' => get_bloginfo( 'name' ),
        'hasValidKey' => peachpay_has_valid_key(),
        'pluginVersion' => PEACHPAY_VERSION
    );
}

define( 'PEACHPAY_ROUTE_BASE', 'peachpay/v1' );

function peachpay_rest_api_init() {
    register_rest_route( PEACHPAY_ROUTE_BASE, '/tax', array(
        'methods' => 'POST',
        'callback' => 'peachpay_tax_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, '/shipping', array(
        'methods' => 'POST',
        'callback' => 'peachpay_shipping_options_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, 'cart/fees', array(
        'methods' => 'POST',
        'callback' => 'peachpay_cart_fees_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, '/variation', array(
        'methods' => 'GET',
        'callback' => 'peachpay_variation_name_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, '/variation/details', array(
        'methods' => 'GET',
        'callback' => 'peachpay_variation_details_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, '/product/(?P<id>\d+)/price', array(
        'methods' => 'GET',
        'callback' => 'peachpay_product_price_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, '/pickup/hours', array(
        'methods' => 'GET',
        'callback' => 'peachpay_pickup_hours_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, '/upload', array(
        'methods' => 'POST',
        'callback' => 'peachpay_upload_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, '/woo-discount-rules/discount/product', array(
        'methods' => 'GET',
        'callback' => 'peachpay_woo_discount_rules_discount_rest',
        'permission_callback' => '__return_true',
    ) );
    register_rest_route( PEACHPAY_ROUTE_BASE, '/plugin', array(
        'methods' => 'GET',
        'callback' => 'peachpay_check_plugin_status',
        'permission_callback' => '__return_true',
    ) );
}

add_action( 'rest_api_init', 'peachpay_rest_api_init');
