<?php
/**
 * Plugin Name: PeachPay for WooCommerce | One-Click Checkout
 * Plugin URI: https://peachpay.app
 * Description: PeachPay is the fastest checkout for WooCommerce.
 * Version: 1.35.0
 * Author: PeachPay, Inc.
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 *
 * WC requires at least: 4.0
 * WC tested up to: 5.2
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
if ( ! is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
    exit;
}

define( 'PEACHPAY_VERSION', get_plugin_data(__FILE__)[ 'Version' ] );

include_once( plugin_dir_path( __FILE__ ) . 'includes/gateway.php' );

function init() {
    add_action( 'wc_ajax_wc_peachpay_create_order', 'peachpay_ajax_create_order' );
}
init();

/**
 * peachpay_api_env determines which environment we are running in so we can call
 * the correct PeachPay API (dev.peachpay.app or prod.peachpay.app).
 */
function peachpay_api_url() {
    $demo_stores = array( 'woo', 'theme1', 'theme2', 'theme3', 'theme4', 'theme5' );
    if (in_array(peachpay_subdomain(get_site_url()), $demo_stores)) {
        return 'https://dev.peachpay.app/';
    }
    if (get_site_url() === 'http://localhost:8000') {
        // TODO: This doesn't work because the server runs in a separate
        // docker-compose project. Need to find a solution for local development.
        // return 'http://localhost:3001/';
        return 'https://dev.peachpay.app/';
    }
    if ( get_option( 'peachpay_options' )[ 'test_mode' ] ) {
        return 'https://dev.peachpay.app/';
    }
    return 'https://prod.peachpay.app/';
}

/**
 * peachpay_subdomain gets the leftmost subdomain out of a URL.
 */
function peachpay_subdomain( $url ) {
    $parsedUrl = parse_url( $url );
    $host = explode('.', $parsedUrl['host']);
    return $host[0];
}

function peachpay_email_us() {
    $body = array(
        'url' => get_site_url(),
        'email' => get_bloginfo('admin_email')
    );
    peachpay_email( $body, 'plugin/activate' );
}

function peachpay_email_merchant_welcome() {
    $body = array(
        'email' => get_bloginfo( 'admin_email' )
    );
    peachpay_email( $body, 'mail/welcome' );
}

function peachpay_email( $body, $endpoint ) {
    $post_body = wp_json_encode( $body );
    $args = array(
        'body' => $post_body,
        'headers' => [ 'Content-Type' => 'application/json' ],
        'httpversion' => '2.0',
        'blocking' => false
    );
    wp_remote_post( peachpay_api_url() . $endpoint, $args );
}

/**
 * peachpay_get_perm asks the merchant that just activated the plugin for
 * permission to access the store's API. This is used to place orders, etc.
 */
function peachpay_get_perm( $plugin ) {
    if ( $plugin != plugin_basename( __FILE__ ) ) {
        // Because we run peachpay_get_perm on the activated_plugin hook, it fires
        // when any plugin is activated, not just ours. Exit if not ours.
        return;
    }
    if ( peachpay_has_valid_key() ) {
        // No need to ask for permission if they already gave permission before.
        // If they have our activation key, that means we have their API keys.
        return;
    }
    $store_url = get_site_url();
    $endpoint = '/wc-auth/v1/authorize';
    $return_url = "https://dashboard.stripe.com/oauth/v2/authorize?response_type=code&client_id=ca_HHK0N5DreIcJJAyqGbeOE77hAZD9gCFg&scope=read_write&state=$store_url&stripe_user[url]=$store_url";
    $params = [
        'app_name' => 'PeachPay',
        'scope' => 'read_write',
        'user_id' => 1,
        'return_url' => $return_url,
        'callback_url' => peachpay_api_url() . 'store-token'
    ];
    $query_string = http_build_query( $params );
    $url = $store_url . $endpoint . '?' . $query_string;
    peachpay_email_us();
    peachpay_email_merchant_welcome();
    exit( wp_redirect( $url ) );
}
add_action( 'activated_plugin', 'peachpay_get_perm' );

function peachpay_admin_notice_onboarding() {
    $logo = plugins_url( 'img/peachpay-full-orange.png', __FILE__ );
    if ( ! get_user_meta( get_current_user_id(), 'peachpay_notice_dismissed' ) && ! peachpay_has_valid_key() ) {
    ?>
    <div class="notice notice-success" style="padding: 1rem; display: flex; justify-content: space-around; align-items: center;">
        <img
            src="<?php echo $logo ?>"
            width="150"
            style="display: block;"
        >
        <p style="width: 70%">
            Welcome to PeachPay!
            <b><a href="https://dashboard.stripe.com/oauth/authorize?response_type=code&client_id=ca_HHK0N5DreIcJJAyqGbeOE77hAZD9gCFg&scope=read_write&state=<?php echo get_site_url(); ?>&stripe_user[url]=<?php echo get_site_url(); ?>">Let's connect your Stripe account.</a></b> Need help? Contact us at <a href="mailto:info@peachpay.app">info@peachpay.app</a>.
        </p>
        <p style="display: inline; float: right;"><a href="?peachpay-notice-dismissed">Don't show this again</a></p>
    </div>
    <?php
    }
}
add_action( 'admin_notices', 'peachpay_admin_notice_onboarding' );

function peachpay_notice_dismissed() {
    if ( isset( $_GET['peachpay-notice-dismissed'] ) ) {
        add_user_meta( get_current_user_id(), 'peachpay_notice_dismissed', 'true', true );
    }
}
add_action( 'admin_init', 'peachpay_notice_dismissed' );

function peachpay_reset_notice_dismissed() {
    delete_user_meta( get_current_user_id(), 'peachpay_notice_dismissed' );
}
register_deactivation_hook( __FILE__, 'peachpay_reset_notice_dismissed' );

/**
 * peachpay_has_valid_key checks if a valid API key has been set.
 */
function peachpay_has_valid_key() {
    // The option is serialized as "1" or "0", so that's why true/false is
    // returned explicitly
    return get_option( 'peachpay_valid_key' ) ? true : false;
}
add_action( 'wp', 'peachpay_has_valid_key' );

function peachpay_load_styles() {
    wp_enqueue_style(
        'peachpay-css',
        plugin_dir_url( __FILE__ ) . 'css/peachpay.css',
        array(),
        peachpay_file_version('css/peachpay.css')
    );
}
add_action( 'wp_enqueue_scripts', 'peachpay_load_styles' );

/**
 * peachpay_load_button_scripts adds the JavaScript files to the page as it
 * loads, and these JavaScript files insert the PeachPay button.
 */
function peachpay_load_button_scripts() {
    // Merchants can use this hook to hide the PeachPay button on certain pages
    if ( function_exists( 'is_product' ) && is_product() ) {
        if ( apply_filters( 'peachpay_hide_button_on_product_page', false ) ) {
            return;
        }
        // We might later turn this into an option in the PeachPay settings
        if (
            get_site_url() === 'https://staging.littletigergifts.co.uk'
            || get_site_url() === 'https://staging.littletigergifts.co.uk/'
            || get_site_url() === 'https://www.littletigergifts.co.uk'
            || get_site_url() === 'https://www.littletigergifts.co.uk/'
        ) {
            return;
        }
    }

    wp_enqueue_script(
        'sentry',
        'https://js.sentry-cdn.com/dd5e3292f8514baa872dcead0794f805.min.js'
    );

    wp_enqueue_script(
        'sentry-init',
        plugin_dir_url( __FILE__ ) . 'js/sentry.js',
        array('sentry'),
        peachpay_file_version('js/sentry.js')
    );

    wp_enqueue_script(
        'pp-translations',
        plugin_dir_url( __FILE__ ) . 'js/translations.js',
        array(),
        peachpay_file_version('js/translations.js')
    );

    wp_enqueue_script(
        'pp-button-product-page',
        plugin_dir_url( __FILE__ ) . 'js/product-page.js',
        array(),
        peachpay_file_version('js/product-page.js')
    );

    wp_enqueue_script(
        'pp-button-cart-page',
        plugin_dir_url( __FILE__ ) . 'js/cart-page.js',
        array(),
        peachpay_file_version('js/cart-page.js')
    );

    wp_enqueue_script(
        'pp-button-core',
        plugin_dir_url( __FILE__ ) . 'js/button.js',
        array( 'pp-translations' ),
        peachpay_file_version('js/button.js')
    );

    $options = get_option( 'peachpay_options' );

    wp_localize_script( 'pp-button-core', 'php_data', array(
        '_wpnonce' => wp_create_nonce( 'woocommerce-process_checkout' ),
        'merchant_name' => get_bloginfo( 'name' ),
        'wp_admin_or_editor' => current_user_can('editor') || current_user_can('administrator'),
        'version' => PEACHPAY_VERSION,
        'num_shipping_zones' => sizeof( WC_Shipping_Zones::get_zones() ),
        'cart' => peachpay_get_cart(),
        'cart_fees' => is_cart() ? WC()->cart->get_fees() : null,
        'cart_coupons' => is_cart() ? WC()->cart->get_coupon_discount_totals() : array(),
        'cart_coupon_names' => is_cart() ? peachpay_cart_coupon_names() : array(),
        // cart_total_tax is only used to fix a very specific bug on tires.pneupress.com where the
        // usual server request must be called twice with cookies in order for it to send back
        // the correct tax amount. Instead of doing something weird like that, we just get the tax
        // right here since for this store it is the same no matter what address.
        'cart_total_tax' => is_cart() ? WC()->cart->get_total_tax() : null,
        'currency' => get_woocommerce_currency(),
        'currency_symbol' => get_woocommerce_currency_symbol(),
        'prices_include_tax' => wc_prices_include_tax(),
        'is_category_page' => is_product_category(),
        'is_cart_page' => is_cart(),
        'wc_cart_url' => wc_get_cart_url(),
        // this one is different from `cart` because it is always the actual cart,
        // whereas the other one is only the product page item if on a product page
        'wc_cart' => peachpay_make_cart_from_WC_cart( WC()->cart->get_cart() ),
        'wc_order_received_url' => wc_get_endpoint_url( 'order-received', '', wc_get_checkout_url() ),
        'wc_prices_include_tax' => wc_prices_include_tax(),
        'wc_store_country_code' => WC()->countries->get_base_country(),
        'file_upload' => isset( $options[ 'file_upload' ] ) ? $options[ 'file_upload' ] : null,
        'test_mode' => isset( $options[ 'test_mode' ] ) ? $options[ 'test_mode' ] : null,
        'email_receipt' => isset ( $options[ 'email_receipt' ] ) ? $options[ 'email_receipt' ] : null,
        'language' => isset( $options[ 'language' ] ) ? $options[ 'language' ] : 'en-US',
        'button_color' => isset( $options[ 'button_color' ] ) ? $options[ 'button_color' ] : null,
        'button_position_product_page' => isset( $options[ 'product_button_position' ] ) ? $options[ 'product_button_position' ] : null,
        'button_position_cart_page' => isset( $options[ 'cart_button_position' ] ) ? $options[ 'cart_button_position' ] : null,
        'button_width_product_page' => isset( $options[ 'button_width_product_page' ] ) ? $options[ 'button_width_product_page' ] : null,
        'button_width_cart_page' => isset( $options[ 'button_width_cart_page' ]) ? $options[ 'button_width_cart_page' ] : null,
        'plugin_woo_discount_rules' => is_plugin_active( 'woo-discount-rules/woo-discount-rules.php' ),
        'plugin_woocommerce_product_addon' => is_plugin_active( 'woocommerce-product-addon/woocommerce-product-addon.php' ),
        'plugin_dc_woocommerce_multi_vendor_name_lookup' => peachpay_get_vendor_map(),
        'plugin_woocommerce_points_and_rewards_active' => is_plugin_active( 'woocommerce-points-and-rewards/woocommerce-points-and-rewards.php' ),
        'plugin_woocommerce_product_bundles_active' => is_plugin_active( 'woocommerce-product-bundles/woocommerce-product-bundles.php' )
    ) );
}
if ( peachpay_has_valid_key() || ( get_option( 'peachpay_options' ) && ( get_option( 'peachpay_options' )[ 'test_mode' ] ?? false ) ) ) {
    add_action( 'wp_enqueue_scripts', 'peachpay_load_button_scripts' );
}

function peachpay_file_version( $file ) {
    return date( "ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . $file ) );
}

function peachpay_get_price( $product ) {
    $price = $product->get_price();
    // Even if the Woo Discount Rules is active, this product may not have a discount
    $maybe_discounted_price = peachpay_woo_discount_rules_get_discount( $product->get_id() );
    if ($maybe_discounted_price) {
        $price = $maybe_discounted_price;
    }
    return $price;
}

/**
 * peachpay_get_cart returns an array of products that are in the "cart". If we
 * are on a single product page, the returned cart is only the single item
 * regardless of the actual contents of the cart.
 */
function peachpay_get_cart() {
    if ( function_exists( 'wc_get_product' ) ) {
        $product = wc_get_product();
        if ( get_site_url() === 'https://www.jogiachamasalo.com' || get_site_url() === 'https://www.jogiachamasalo.com/' ) {
            $product = wc_get_product( 160 );
        }
        if ( $product ) {
            return peachpay_product_page_cart( $product );
        }
    }
    if ( is_null( WC()->cart ) ) {
        return array();
    }
    return peachpay_make_cart_from_WC_cart( WC()->cart->get_cart() );
}

function peachpay_product_page_cart( $product ) {
    if ( ! isset( $product ) || ! $product ) {
        return array();
    }

    // Grouped products
    $child_ids = $product->get_children();
    if ( $product->get_type() === 'grouped' && $child_ids ) {
        $children = array();
        foreach ( $child_ids as $id ) {
            array_push( $children, wc_get_product( $id ) );
        }
        return peachpay_make_product_page_cart( $children, true );
    }

    // WooCommerce Product Bundles plugin
    if ( $product->get_type() === 'bundle' ) {
        $cart = array();
        array_push( $cart, $product );
        foreach ( $product->get_bundled_items() as $item ) {
            array_push( $cart, wc_get_product( $item->get_product() ) );
        }
        return peachpay_make_product_page_cart( $cart, false, true );
    }

    return peachpay_make_product_page_cart( array( $product ) );
}

function peachpay_make_product_page_cart( $products, $is_grouped = false, $is_bundled = false ) {
    $cart = array();

    foreach ( $products as $product ) {
        $price = peachpay_get_price( $product );
        // People might not buy one of the grouped items, so allow 0 quantity
        $quantity = $is_grouped ? 0 : null;

        // The first check is for whether is group of products **contains** a parent bundle product,
        // and the second check is for whether the current product in the loop **is** the
        // parent bundle product.
        if ( $is_bundled && $product->get_type() !== 'bundle' ) {
            // Only the parent in a bundle has a price
            $price = 0;
            $quantity = 1;
        }

        $item = array(
            'product_id' => $product->get_id(),
            'variation_id' => $product->get_id(), // no variation support on grouped products for now
            'name' => $product->get_name(),
            'price' => $price,
            'quantity' => $quantity,
            'virtual' => $product->is_virtual(),
            'subtotal' => strval( $price ), // subtotal and total are only relevant for what shows up in the order dashboard
            'total' => strval( $price )
        );

        if ( is_plugin_active( 'dc-woocommerce-multi-vendor/dc_product_vendor.php' ) ) {
            $item['vendor_id'] = get_wcmp_product_vendors( $product->get_id() )->id;
            $item['vendor_name'] = get_wcmp_product_vendors( $product->get_id() )->user_data->data->display_name;
        }

        // Don't show the price on the order dashboard for items that are part of a bundle
        if ( $is_bundled && $product->get_type() !== 'bundle' ) {
            $item['subtotal'] = '0';
            $item['total'] = '0';
            $item['is_part_of_bundle'] = true;
        }

        array_push( $cart, $item );
    }

    return $cart;
}

function peachpay_make_cart_from_WC_cart( $items ) {
    $cart = array();
    foreach ( $items as $item ) {
        $product = $item['data'];
        $item_to_add = array(
            'product_id' => $product->get_id(),
            'variation_id' => $product->get_id(), // Why? WC_Product::get_variation_id is deprecated since version 3.0. Use WC_Product::get_id(). It will always be the variation ID if this is a variation.
            'name' => peachpay_product_name_always_with_variation( $product ) ,
            'price' => peachpay_get_price( $product ),
            'quantity' => $item['quantity'],
            'virtual' => $product->is_virtual(),
            'subtotal' => strval( peachpay_get_price( $product ) ), // subtotal and total are only relevant for what shows up in the order dashboard
            'total' => strval( peachpay_get_price( $product ) )
        );
        if ( function_exists( 'wc_pb_is_bundled_cart_item' ) && wc_pb_is_bundled_cart_item( $item ) ) {
            $item_to_add['is_part_of_bundle'] = true;
            $item_to_add['quantity'] = 1;
            $item_to_add['price'] = 0;
            $item_to_add['subtotal'] = '0';
            $item_to_add['total'] = '0';
        }
        if ( is_plugin_active( 'dc-woocommerce-multi-vendor/dc_product_vendor.php' ) ) {
            $item_to_add['vendor_id'] = get_wcmp_product_vendors( $product->get_id() )->id;
            $item_to_add['vendor_name'] = get_wcmp_product_vendors( $product->get_id() )->user_data->data->display_name;
        }
        if ( is_plugin_active( 'woocommerce-product-addon/woocommerce-product-addon.php' ) && isset( $item['ppom'] ) ) {
            // meta_data are key value pairs that are seen on each product
            // in the order dashboard for a particular order
            $item_to_add['meta_data'] = array();
            foreach ( $item['ppom']['fields'] as $key => $value ) {
                // id is produced by the plugin, but we don't need it
                if ( $key === 'id' ) {
                    continue;
                }
                array_push($item_to_add['meta_data'], array(
                    'key' => $key,
                    'value' => $value,
                ));
            }
        }
        array_push( $cart, $item_to_add );
    }
    return $cart;
}

/**
 * Get the full product name even if the filter
 * woocommerce_product_variation_title_include_attributes has been set to not
 * include the variation in the title.
 *
 * add_filter( 'woocommerce_product_variation_title_include_attributes', '__return_false' );
 *
 * This is used for the cart page checkout window to display the variation as part of the title.
 *
 * This pretty much takes the code from the internal function generate_product_title
 * from woocommerce/includes/data-stores/class-wc-product-variation-data-store-cpt.php
 * and removes the filter part.
 *
 * If this is a simple product with no variations, it returns the base name.
 */
function peachpay_product_name_always_with_variation( $id ) {
    $product = wc_get_product( $id );
    if ( ! $product ) {
        return '';
    }
    if ( $product instanceof WC_Product_Variation ) {
        $separator = apply_filters( 'woocommerce_product_variation_title_attributes_separator', ' - ', $product );
        return get_post_field( 'post_title', $product->get_parent_id() ) . $separator . wc_get_formatted_variation( $product, true, false );
    }
    return $product->get_name();
}

function peachpay_woocommerce_ajax_add_to_cart() {
    $product_id = apply_filters( 'peachpay_woocommerce_add_to_cart_product_id', absint( $_POST['product_id'] ) );
    $quantity = empty( $_POST['quantity'] ) ? 1 : wc_stock_amount( $_POST['quantity'] );
    $variation_id = absint( $_POST['variation_id'] );
    $passed_validation = apply_filters( 'peachpay_woocommerce_add_to_cart_validation', true, $product_id, $quantity );
    $product_status = get_post_status( $product_id );
    if (
        $passed_validation
        && ! peachpay_product_in_cart( $product_id, $variation_id )
        && WC()->cart->add_to_cart( $product_id, $quantity, $variation_id )
        && 'publish' === $product_status
    ) {
        do_action( 'peachpay_woocommerce_ajax_added_to_cart', $product_id );
        if ( 'yes' === get_option( 'peachpay_woocommerce_cart_redirect_after_add' ) ) {
            wc_add_to_cart_message( array( $product_id => $quantity ), true );
        }
        WC_AJAX::get_refreshed_fragments();
    } else {
        $data = array(
            'error' => true,
            'product_url' => apply_filters( 'peachpay_woocommerce_cart_redirect_after_error', get_permalink( $product_id ), $product_id )
        );
        echo wp_send_json( $data );
    }
    wp_die();
}
add_action('wp_ajax_peachpay_woocommerce_ajax_add_to_cart', 'peachpay_woocommerce_ajax_add_to_cart');
add_action('wp_ajax_nopriv_peachpay_woocommerce_ajax_add_to_cart', 'peachpay_woocommerce_ajax_add_to_cart');

function peachpay_product_in_cart( $product_id, $variation_id ) {
    foreach( WC()->cart->get_cart() as $cart_item_key => $values ) {
        $_product = $values['data'];
        if( $product_id === $_product->get_id() || $variation_id === $_product->get_id() ) {
            return true;
        }
    }
    return false;
}

function peachpay_cart_coupon_names() {
    if ( ! is_cart() ) {
        return;
    }
    $coupons = WC()->cart->get_coupon_discount_totals();
    $names = array();
    foreach ( $coupons as $coupon_code => $amount ) {
        $match = array();
        preg_match( '/(?<=wc_)([^\d])*(?=_\d+)/', $coupon_code, $match );
        $names[$coupon_code] = ucfirst( str_replace( '_', ' ', $match[0] ) );
    }
    return $names;
}

function peachpay_get_vendor_map() {
    if ( ! is_plugin_active( 'dc-woocommerce-multi-vendor/dc_product_vendor.php' ) ) {
        return array();
    }
    $vendors = get_wcmp_vendors();
    $vendor_map = array();
    foreach ( $vendors as $vendor ) {
        $vendor_map[$vendor->id] = $vendor->user_data->data->display_name;
    }
    return $vendor_map;
}

include_once( plugin_dir_path( __FILE__ ) . 'includes/settings.php' );
include_once( plugin_dir_path( __FILE__ ) . 'includes/rest_api.php' );

// Add the PeachPay settings link in the plugin list, next to "Deactivate"
$plugin_basename = plugin_basename( __FILE__ );
add_filter( "plugin_action_links_$plugin_basename", 'peachpay_add_settings_link' );
